'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Chart = exports.defaults = exports.Scatter = exports.Bubble = exports.Polar = exports.Radar = exports.HorizontalBar = exports.Bar = exports.Line = exports.Pie = exports.Doughnut = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _preact = require('preact');

var _chart = require('chart.js');

var _chart2 = _interopRequireDefault(_chart);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

class ChartComponent extends _preact.Component {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.handleOnClick = event => {
      const instance = this.chart_instance;

      const {
        getDatasetAtEvent,
        getElementAtEvent,
        getElementsAtEvent,
        onElementsClick
      } = this.props;

      getDatasetAtEvent && getDatasetAtEvent(instance.getDatasetAtEvent(event), event);
      getElementAtEvent && getElementAtEvent(instance.getElementAtEvent(event), event);
      getElementsAtEvent && getElementsAtEvent(instance.getElementsAtEvent(event), event);
      onElementsClick && onElementsClick(instance.getElementsAtEvent(event), event); // Backward compatibility
    }, this.ref = element => {
      this.element = element;
    }, _temp;
  }

  componentWillMount() {
    this.chart_instance = undefined;
  }

  componentDidMount() {
    this.renderChart();
  }

  componentDidUpdate() {
    if (this.props.redraw) {
      this.chart_instance.destroy();
      this.renderChart();
      return;
    }

    this.updateChart();
  }

  shouldComponentUpdate(nextProps) {
    const {
      redraw,
      type,
      options,
      plugins,
      legend,
      height,
      width
    } = this.props;

    if (nextProps.redraw === true) {
      return true;
    }

    if (height !== nextProps.height || width !== nextProps.width) {
      return true;
    }

    if (type !== nextProps.type) {
      return true;
    }

    if (!(0, _isEqual2.default)(legend, nextProps.legend)) {
      return true;
    }

    if (!(0, _isEqual2.default)(options, nextProps.options)) {
      return true;
    }

    const nextData = this.transformDataProp(nextProps);

    if (!(0, _isEqual2.default)(this.shadowDataProp, nextData)) {
      return true;
    }

    return !(0, _isEqual2.default)(plugins, nextProps.plugins);
  }

  componentWillUnmount() {
    this.chart_instance.destroy();
  }

  transformDataProp(props) {
    const { data } = props;
    if (typeof data == 'function') {
      const node = this.element;
      return data(node);
    } else {
      return data;
    }
  }

  // Chart.js directly mutates the data.dataset objects by adding _meta proprerty
  // this makes impossible to compare the current and next data changes
  // therefore we memoize the data prop while sending a fake to Chart.js for mutation.
  // see https://github.com/chartjs/Chart.js/blob/master/src/core/core.controller.js#L615-L617
  memoizeDataProps() {
    if (!this.props.data) {
      return;
    }

    const data = this.transformDataProp(this.props);

    this.shadowDataProp = _extends({}, data, {
      datasets: data.datasets && data.datasets.map(set => {
        return _extends({}, set);
      })
    });

    return data;
  }

  updateChart() {
    const { options } = this.props;

    const data = this.memoizeDataProps(this.props);

    if (!this.chart_instance) return;

    if (options) {
      this.chart_instance.options = _chart2.default.helpers.configMerge(this.chart_instance.options, options);
    }

    // Pipe datasets to chart instance datasets enabling
    // seamless transitions
    let currentDatasets = this.chart_instance.config.data && this.chart_instance.config.data.datasets || [];
    const nextDatasets = data.datasets || [];

    // use the key provider to work out which series have been added/removed/changed
    const currentDatasetKeys = currentDatasets.map(this.props.datasetKeyProvider);
    const nextDatasetKeys = nextDatasets.map(this.props.datasetKeyProvider);
    const newDatasets = nextDatasets.filter(d => currentDatasetKeys.indexOf(this.props.datasetKeyProvider(d)) === -1);

    // process the updates (via a reverse for loop so we can safely splice deleted datasets out of the array
    for (let idx = currentDatasets.length - 1; idx >= 0; idx -= 1) {
      const currentDatasetKey = this.props.datasetKeyProvider(currentDatasets[idx]);
      if (nextDatasetKeys.indexOf(currentDatasetKey) === -1) {
        // deleted series
        currentDatasets.splice(idx, 1);
      } else {
        const retainedDataset = (0, _find2.default)(nextDatasets, d => this.props.datasetKeyProvider(d) === currentDatasetKey);
        if (retainedDataset) {
          // update it in place if it is a retained dataset
          currentDatasets[idx].data.splice(retainedDataset.data.length);
          retainedDataset.data.forEach((point, pid) => {
            currentDatasets[idx].data[pid] = retainedDataset.data[pid];
          });
          const { data } = retainedDataset,
                otherProps = _objectWithoutProperties(retainedDataset, ['data']);
          currentDatasets[idx] = _extends({
            data: currentDatasets[idx].data
          }, currentDatasets[idx], otherProps);
        }
      }
    }
    // finally add any new series
    newDatasets.forEach(d => currentDatasets.push(d));
    const { datasets } = data,
          rest = _objectWithoutProperties(data, ['datasets']);

    this.chart_instance.config.data = _extends({}, this.chart_instance.config.data, rest);

    this.chart_instance.update();
  }

  renderChart() {
    const { options, legend, type, redraw, plugins } = this.props;
    const node = this.element;
    const data = this.memoizeDataProps();

    if (typeof legend !== 'undefined' && !(0, _isEqual2.default)(ChartComponent.defaultProps.legend, legend)) {
      options.legend = legend;
    }

    this.chart_instance = new _chart2.default(node, {
      type,
      data,
      options,
      plugins
    });
  }

  render() {
    const { height, width, onElementsClick } = this.props;

    return (0, _preact.h)('canvas', {
      ref: this.ref,
      height: height,
      width: width,
      onClick: this.handleOnClick
    });
  }
}

ChartComponent.getLabelAsKey = d => d.label;

ChartComponent.defaultProps = {
  legend: {
    display: true,
    position: 'bottom'
  },
  type: 'doughnut',
  height: 150,
  width: 300,
  redraw: false,
  options: {},
  datasetKeyProvider: ChartComponent.getLabelAsKey
};
exports.default = ChartComponent;
class Doughnut extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'doughnut'
    }));
  }
}

exports.Doughnut = Doughnut;
class Pie extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'pie'
    }));
  }
}

exports.Pie = Pie;
class Line extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'line'
    }));
  }
}

exports.Line = Line;
class Bar extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'bar'
    }));
  }
}

exports.Bar = Bar;
class HorizontalBar extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'horizontalBar'
    }));
  }
}

exports.HorizontalBar = HorizontalBar;
class Radar extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'radar'
    }));
  }
}

exports.Radar = Radar;
class Polar extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'polarArea'
    }));
  }
}

exports.Polar = Polar;
class Bubble extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'bubble'
    }));
  }
}

exports.Bubble = Bubble;
class Scatter extends _preact.Component {
  render() {
    return (0, _preact.h)(ChartComponent, _extends({}, this.props, {
      ref: ref => this.chart_instance = ref && ref.chart_instance,
      type: 'scatter'
    }));
  }
}

exports.Scatter = Scatter;
const defaults = exports.defaults = _chart2.default.defaults;
exports.Chart = _chart2.default;